package descartes.kieker4dql.dqlconnector.configgenerator;

import java.io.File;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XMLGenerator {

	private static final String XML_ELEMENT_ASPECTJ = "aspectj";
	private static final String XML_ELEMENT_WEAVER = "weaver";
	private static final String XML_ATTRIBUTE_OPTIONS = "options";
	private static final String XML_ELEMENT_INCLUDE = "include";
	private static final String XML_ELEMENT_EXCLUDE = "exclude";
	private static final String XML_ATTRIBUTE_WITHIN = "within";
	private static final String XML_ELEMENT_ASPECTS = "aspects";
	private static final String XML_ELEMENT_ASPECT = "aspect";
	private static final String XML_ATTRIBUTE_NAME = "name";

	private static DocumentBuilderFactory docFactory;
	private static DocumentBuilder docBuilder;
	private static Document doc;
	private static TransformerFactory transformerFactory;
	private static Transformer transformer;

	private static void init() throws ParserConfigurationException, TransformerConfigurationException {
		// Documentbuilder declarations
		DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docFactory.newDocumentBuilder();

		// Document element
		Document doc = docBuilder.newDocument();
		// declare transformers
		transformerFactory = TransformerFactory.newInstance();
		transformer = transformerFactory.newTransformer();
	}

	public static void createXml(String path, String weaverOptions, List<String> includeOptions,
			List<String> excludeOptions, List<String> aspectNames)
			throws ParserConfigurationException, TransformerException {

		init();

		// <aspectj>
		Element aspectjElement = appendElement(XML_ELEMENT_ASPECTJ);
		// <weaver options="">
		Element weaverElement = appendElement(aspectjElement, XML_ELEMENT_WEAVER, XML_ATTRIBUTE_OPTIONS, weaverOptions);
		// <include withhin="">
		for (String includeOption : includeOptions) {
			Element includeElement = appendElement(weaverElement, XML_ELEMENT_INCLUDE, XML_ATTRIBUTE_WITHIN,
					includeOption);
		}
		// <exclude withhin="">
		for (String excludeOption : excludeOptions) {
			Element includeElement = appendElement(weaverElement, XML_ELEMENT_EXCLUDE, XML_ATTRIBUTE_WITHIN,
					excludeOption);
		}

		// <aspects>
		Element aspectsElement = appendElement(aspectjElement, XML_ELEMENT_ASPECTS, "", "");

		// <aspect name ="">
		for (String aspectName : aspectNames) {
			Element aspectElement = appendElement(aspectsElement, XML_ELEMENT_ASPECT, XML_ATTRIBUTE_NAME, aspectName);
		}

		DOMSource source = new DOMSource(doc);
		StreamResult result = createFile(path);

		transformer.transform(source, result);

	}

	private static Element appendElement(Element parent, String elementName, String attributeName,
			String attributeValue) {
		Element element = createElement(elementName, attributeName, attributeValue);
		parent.appendChild(element);
		return element;
	}

	private static Element appendElement(String elementName) {
		Element element = createElement(elementName, "", "");
		doc.appendChild(element);
		return element;
	}

	private static Element createElement(String elementName, String attributeName, String attributeValue) {
		Element element = doc.createElement(elementName);
		if (!attributeName.equals("")) {
			Attr attr = createAttribute(attributeName);
			attr.setValue(attributeValue);
			element.appendChild(attr);
		}
		return element;
	}

	private static Attr createAttribute(String attributeName) {

		Attr attr = doc.createAttribute(attributeName);
		return attr;

	}

	private static StreamResult createFile(String path) {
		StreamResult result = new StreamResult(new File(path));
		return result;
	}

}
